---
дата: 2024-07-26
tags:
  - "#dev"
---
[TreeSheets - Проект](https://github.com/aardappel/treesheets)
[Указания к переводу](https://github.com/aardappel/treesheets/blob/master/TS/translations/readme_translations.txt)

Каталог Проекта **TS/translations** содержит *переводы пользовательского интерфейса* TreeSheets на разные языки.

Для работы над переводом в *Linux/OSX* потребуются команды `xgettext msginit msgmerge msgfmt` из пакета **gettext** (установите его).
В *Windows* это тоже возможно, например: https://mlocati.github.io/articles/gettext-icv-windows.html

**Gettext** - библиотека проекта GNU, предназначенная для унификации (и минимализации) работ по переводу пользовательского интерфейса и сообщений для конечного пользователя.[ Краткое руководство по Gettext](https://www.labri.fr/perso/fleury/posts/programming/a-quick-gettext-tutorial.html)
> Gettext работает как с *интернационализацией* (**i18n**), так и с *локализацией* (**L10n**) - двумя разными технологиями, связанными с адаптацией приложений и продуктов для международного использования. [Отличия i18n и L10n](https://wp-yoda.com/adaptacziya-programmnyh-produktov/otlichiya-i18n-i-i10n/).

## Алгоритм локализации

### Первоначальный алгоритм получения файлов перевода

```
          xgettext          msginit            msgfmt
../help.c -------> help.pot ------> ru/help.po -----> ru/help.mo
```
1) Строки предназначенные для перевода вытягиваются из исходных файлов в файл шаблонна **.pot**.
2) Из *pot*-файла шаблона создаются **po**-файлы, предназначенные для переводов на другие языки (в нашем случае - ru).
   Переводчик выполняет перевод в файле **.po**.
3) *po*-файл с переводами преобразуется в машиночитаемый **.mo** (для программы).

### Если в исходный код были добавлены ​​новые строки

В этом случае выполняется слияние изменений, которое в общем следует алгоритму:
```
         xgettext -j        msgmerge           msgfmt
../help.c -------> help.pot ------> ru/help.po -----> ru/help.mo
```
1) *Новые строки* предназначенные для перевода *вытягиваются* из исходных файлов и *сливаются в существующий файл* шаблонна **.pot** (доп. опция **-j**).
2) Из актуализированного pot-файла шаблона **po**-файлы переводов *обновляются* (дополняются) новыми строками.
   Переводчик выполняет *перевод новых строк* в обновлённом файле **.po**.
3) *po*-файл с переводами преобразуется в машиночитаемый **.mo** (для программы).

## Процесс получения перевода

### 1 Основной Шаблон (pot-файл)

Первоначально *создаётся (обновляется) **основной Шаблон** - **.pot**-файл* (*Portable Object Template* - Шаблон Портативного/Переносимого Объекта) со всеми строками для перевода, извлеченными непосредственно из исходных файлов. Данный файл шаблона используется для [[TreeSheets#2 Создание файл перевода (po-файл)|создания файлов перевода на другие языки]] (**.po**-*файлы*).

> *Обычно* файл Шаблона *создаётся и обновляется разработчиком*.

#### Создание
Чтобы *создать* основной файл шаблона `/TS/translations/ts.pot`, находясь в каталоге `src/`, запустите `genpot.bat` или команду:
```sh
xgettext --keyword=_ --sort-output --no-location -o ../TS/translations/ts.pot myframe.h document.h system.h
```
#### Обновление
Чтобы *обновить* файла шаблона (*после изменения исходников* в новой версии программы):
```sh
xgettext -j --keyword=_ --sort-output --no-location -o ../TS/translations/ts.pot myframe.h document.h system.h
```


### 2.1 Создание файл перевода (po-файл)

Для создания файлов перевода на другие языки - **.po**-файлы (*Portable Object* - Портативный/Переносимый Объект) используется ранее созданный [[TreeSheets#1 Основной Шаблон (pot-файл)|файл шаблона]] (*.pot-файл*).

Чтобы создать перевод для нового языка, *внутри каталога `TS/translations`* (там где `ts.pot`) выполните команду:
```sh
msginit --input ts.pot --locale=lang[.ENCODING] \
--output=lang/ts.po
```
заменив «**lang**»:
- либо на *двухбуквенный код языка* ISO 639-1 (например, «it»),
- либо на *5-значную комбинацию*: код языка ISO 639-1 + код страны ISO 3166-1 (например: «ru_RU», «ru_UA») для языка перевода,-
а **ENCODING** - на используемую кодировку (обычно *UTF-8*):
```sh
mkdir ru_RU   # языковой подкаталог должен существовать!
msginit --input ts.pot --locale=ru_RU.UTF-8 --output=ru_RU/ts.po
```
...команда создаст **po**-*файл перевода*, который предоставляют переводчику для выполнения перевода на другой язык.

#### 2.1b Если обновлён pot-файл шаблона

[[TreeSheets#Обновление|Обновление pot-файла]] шаблона *обычно выполняется разработчиком* после изменения исходников в новой версии программы.

*Чтобы* объединить существующий перевод (в *po*-файле) с новыми строками из изменённого! *pot*-файла шаблона (т.е. *обновить* **po**-*файл языкового перевода*), запустите из каталога `ru_RU` файл `merge.sh`, или просто выполните команду:
```sh
msgmerge --update --backup=numbered ts.po ../ts.pot
```
После этого - *переведите новые строки в `.po`-файле*...


### 2.2 Перевод

**.po**-*файл перевода* необходимо заполнить переводами строк: т.е. в файле **ts.po** необходимо из строк "msgid" создавать переводы на русский и вставлять в строки "msgstr", например:

```
msgid "Switch to &next file/tab"
msgstr "Перейти к следующей вкладке"

msgid "Switch to &next file/tab\tCTRL+TAB"
msgstr "Перейти к следующей вкладке\tCTRL+TAB"
```

Это можно делать и врукопашную... но существуют утилиты для переводчиков, облегчающие редактирование перевода, например:

#### Poedit

[Poedit](https://poedit.net) -бесплатный и открытый кросс-платформенный инструмент редактирования каталогов локализации для *gettext*.
> При сохранении .po-файла Poedit *пересобирает* и соответствующий *.mo -файл*.

[Расширения формата PO в Poedit](https://github.com/vslavik/poedit/wiki/PO-Extensions)
[Справка по Poedit](https://poedit.net/trac/wiki/Doc)


### 3 Машиночитаемый Объект (.mo-файл) - сборка

> [!important] [[TreeSheets#Poedit|Poedit]] при сохранении .po-файла *пересобирает* и соответствующий *.mo -файл*.

Чтобы перекомпилировать языковой *po*-файл в машиночитаемый **.mo**-*файл* (*Machine Object* - Машинный объект), запустите из каталога `ru_RU` файл `compile.sh`, или просто выполните команду:
```sh
msgfmt --output-file=ts.mo ts.po
```

---
## Extras

### Оперативная визуализация процесса перевода

Для этого поместите ссылку на каталог ru_RU (в котором выполняете перевод)  в  соответствующий каталог программы.
Для **flatpak**:
```sh
ln -s .../GIT/TreeSheets/TS/translations/ru_RU /var/lib/flatpak/app/com.strlen.TreeSheets/current/active/files/share/locale/ru_RU
```
... и перезапустите программу.


### & в Меню (F10)

**&** - это так называемые **мнемоники / акселераторы** (*mnemonics / accelerators*) , которые используются в пунктах меню для навигации по меню с помощью клавиш (см. например, https://www.wxwidgets.org/docs/tutorials/using-mnemonics/ для более подробной информации - https://docs.wxwidgets.org/latest/classwx_menu_item.html#a8b0517fb35e3eada66b51568aa87f261).
Символ **&** предваряет букву из строки элемента управления окном (строки меню, кнопки и т. д.), который вы можете активировать с помощью клавиатуры - нажав соответствующую клавишу. Например для пункта меню "&Отменить" будет достаточно нажать клавишу с буквой "О".
Для сообщений - это не обязательно, поэтому некоторые строки содержат &, а другие - нет.
При переводе, вы можете поместить & на другие буквы (иначе чем в английской строке), если это кажется разумным.
Если буква в одном меню используется несколько раз - нажатия на неё приведут к циклическому перемещению по соответствующим пунктам меню (без активации пункта меню).

